@@ -4,10 +4,10 @@
  * are too numerous to list here. Please refer to the COPYRIGHT
  * file distributed with this source distribution.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -37,7 +36,7 @@ static const TinselGameDescription gameDescriptions[] = {
 
 	// ==== Discworld 1 early (TinselV0) entries ==============================
 
-	{	// Floppy Demo V0 from http://www.adventure-treff.de/specials/dl_demos.php
+	{	// Floppy Demo V0 from https://web.archive.org/web/20100415160943/http://www.adventure-treff.de/specials/dl_demos.php
 		{
 			"dw",
 			"Floppy Demo",
@@ -61,7 +60,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD Demo",
 			AD_ENTRY2s("dw.gra", "ef5a2518c9e205f786f5a4526396e661", 781676,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformDOS,
 			ADGF_DEMO | ADGF_CD,
@@ -80,7 +79,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			{
 				{"dw.scn", 0, "cfc40a8d5d476a1c9d3abf826fa46f8c", 1272686},
 				{"english.txt", 0, "c69b5d2067e9114a63569a61e9a82faa", 228878},
-				{"english.smp", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::EN_ANY,
@@ -101,10 +100,10 @@ static const TinselGameDescription gameDescriptions[] = {
 			"Floppy",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::FR_FRA,
@@ -124,10 +123,10 @@ static const TinselGameDescription gameDescriptions[] = {
 			"Floppy",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::DE_DEU,
@@ -147,10 +146,10 @@ static const TinselGameDescription gameDescriptions[] = {
 			"Floppy",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::IT_ITA,
@@ -170,10 +169,10 @@ static const TinselGameDescription gameDescriptions[] = {
 			"Floppy",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::ES_ESP,
@@ -208,7 +207,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.gra", "c8808ccd988d603dd35dff42013ae7fd", 781656,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformDOS,
 			ADGF_CD,
@@ -225,7 +224,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.gra", "ef05bbd2a754bd11a2e87bcd84ab5ccf", 781864,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformDOS,
 			ADGF_CD,
@@ -261,7 +260,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
 				{"english.txt", 0, "15f0703f85477d7fab4280bf938b61c1", 237774},
-				{"english.smp", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::IT_ITA,
@@ -282,11 +281,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"english.smp", 0, NULL, -1},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::FR_FRA,
@@ -306,11 +305,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"english.smp", 0, NULL, -1},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::DE_DEU,
@@ -329,11 +328,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"english.smp", 0, NULL, -1},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::IT_ITA,
@@ -352,11 +351,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw.gra", 0, "c8808ccd988d603dd35dff42013ae7fd", 781656},
-				{"english.smp", 0, NULL, -1},
-				{"french.txt", 0, NULL, -1},
-				{"german.txt", 0, NULL, -1},
-				{"italian.txt", 0, NULL, -1},
-				{"spanish.txt", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
+				{"french.txt", 0, NULL, AD_NO_SIZE},
+				{"german.txt", 0, NULL, AD_NO_SIZE},
+				{"italian.txt", 0, NULL, AD_NO_SIZE},
+				{"spanish.txt", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::ES_ESP,
@@ -375,7 +374,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.scn", "70955425870c7720d6eebed903b2ef41", 776188,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformDOS,
 			ADGF_CD,
@@ -392,7 +391,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.scn", "759d1374b4f02af6d52fc07c96679936", 770780,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::HE_ISR,
 			Common::kPlatformDOS,
 			ADGF_CD,
@@ -481,7 +480,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			_s("Saturn CD version is not yet supported"),
 			AD_ENTRY2s("dw.scn", "6803f293c88758057cc685b9437f7637", 382248,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformSaturn,
 			ADGF_CD,
@@ -498,7 +497,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.scn", "114643df0d1f1530a0a9c5d4e38917bc", 1268553,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformMacintosh,
 			ADGF_CD,
@@ -515,7 +514,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.scn", "cfc40a8d5d476a1c9d3abf826fa46f8c", 1265532,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformMacintosh,
 			ADGF_CD,
@@ -527,6 +526,24 @@ static const TinselGameDescription gameDescriptions[] = {
 		TINSEL_V1,
 	},
 
+	{	// Mac Japanese CD
+		{
+			"dw",
+			"CD",
+			AD_ENTRY2s("dw.scn", "55a11596b16130027fb28d8c203655a8", 1430841,
+					   "english.smp", NULL, AD_NO_SIZE),
+			Common::JA_JPN,
+			Common::kPlatformMacintosh,
+			ADGF_CD | ADGF_UNSTABLE,
+			GUIO0()
+		},
+		GID_DW1,
+		0,
+		GF_SCNFILES | GF_ENHANCED_AUDIO_SUPPORT,
+		TINSEL_V1,
+	},
+
+
 	{	// German CD re-release "Neon Edition"
 		// Note: This release has ENGLISH.TXT (with german content) instead of GERMAN.TXT
 		{
@@ -551,7 +568,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			{
 				{"dw.scn", 0, "133041bde59d05c1bf084fd6f1bdce4b", 776524},
 				{"english.txt", 0, "f73dcbd7b136b37c2adf7c9448ea336d", 231821},
-				{"english.smp", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::RU_RUS,
@@ -570,7 +587,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw",
 			"CD",
 			AD_ENTRY2s("dw.scn", "36795d539e290838fa1d4c11789cb142", 776188,
-					   "english.smp", NULL, -1),
+					   "english.smp", NULL, AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformDOS,
 			ADGF_CD,
@@ -589,7 +606,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			{
 				{"dw.scn", 0, "133041bde59d05c1bf084fd6f1bdce4b", 776524},
 				{"english.txt", 0, "317542cf2e50106d9c9421ddcf821e22", 221656},
-				{"english.smp", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::RU_RUS,
@@ -610,7 +627,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			{
 				{"dw.scn", 0, "fa169d2c98660215ebd84b49c1899eef", 776396},
 				{"english.txt", 0, "c1a53eb7ec812689dab70e2bb22cf2ab", 224151},
-				{"english.smp", 0, NULL, -1},
+				{"english.smp", 0, NULL, AD_NO_SIZE},
 				AD_LISTEND
 			},
 			Common::PL_POL,
@@ -626,17 +643,53 @@ static const TinselGameDescription gameDescriptions[] = {
 
 	// ==== Discworld 2 entries ===============================================
 	// Note: All Discworld 2 versions are CD only, therefore we don't add the ADGF_CD flag
+#define DISCWORLD2_GUIOPTIONS GUIO2(GUIO_NOASPECT, GAMEOPTION_CROP_HEIGHT_480_TO_432)
 
-	{	// English Discworld 2 demo
+	{	// English Discworld 2 demo (dw2-win-demo-en)
 		{
 			"dw2",
 			"Demo",
 			AD_ENTRY2s("dw2.scn",		"853ab998f5136b69bc586991175d6eeb", 4231121,
 					   "english.smp",	"b5660a0e031cb4710bcb0ef5629ea61d", 28562357),
 			Common::EN_ANY,
-			Common::kPlatformDOS,
+			Common::kPlatformWindows,
 			ADGF_DEMO,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
+		},
+		GID_DW2,
+		0,
+		GF_SCNFILES,
+		TINSEL_V2,
+	},
+
+	{	// English Discworld 2 demo (second Windows demo: dw2-win-demo-2-en)
+		{
+			"dw2",
+			"",
+			AD_ENTRY3s("dw2.scn",     "3f24abb61a058f8faeac7c0768cf21fc", 4224921,
+					   "english.smp", "b5660a0e031cb4710bcb0ef5629ea61d", 31360342,
+					   "english.txt", "f17e10eccac0fb2d1fea489a951da266", 283144),
+			Common::EN_ANY,
+			Common::kPlatformWindows,
+			ADGF_UNSUPPORTED | ADGF_DEMO,
+			DISCWORLD2_GUIOPTIONS
+		},
+		GID_DW2,
+		0,
+		GF_SCNFILES,
+		TINSEL_V2,
+	},
+
+	{	// English (US) Discworld 2 demo (DOS demo: dw2-dos-demo-en)
+		{
+			"dw2",
+			"",
+			AD_ENTRY2s("dw2.scn",     "05beafadd26562d708f68194d337b2cb", 103221,
+					   "us.smp",      "b5660a0e031cb4710bcb0ef5629ea61d", 28320582),
+			Common::EN_USA,
+			Common::kPlatformDOS,
+			ADGF_UNSUPPORTED | ADGF_DEMO,
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -649,11 +702,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw2",
 			"CD",
 			AD_ENTRY2s("dw2.scn", "c6d15ce9720a9d8fef06e6582dcf3f34", 103593,
-					   "english1.smp", NULL, -1),
+					   "english1.smp", NULL, AD_NO_SIZE),
 			Common::EN_GRB,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -666,11 +719,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw2",
 			"CD",
 			AD_ENTRY2s("dw2.scn", "c6d15ce9720a9d8fef06e6582dcf3f34", 103593,
-					   "us1.smp", NULL, -1),
+					   "us1.smp", NULL, AD_NO_SIZE),
 			Common::EN_USA,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -683,11 +736,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw2",
 			"CD",
 			AD_ENTRY2s("dw2.scn", "c6d15ce9720a9d8fef06e6582dcf3f34", 103593,
-					   "french1.smp", NULL, -1),
+					   "french1.smp", NULL, AD_NO_SIZE),
 			Common::FR_FRA,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -700,11 +753,11 @@ static const TinselGameDescription gameDescriptions[] = {
 			"dw2",
 			"CD",
 			AD_ENTRY2s("dw2.scn", "c6d15ce9720a9d8fef06e6582dcf3f34", 103593,
-					   "german1.smp", NULL, -1),
+					   "german1.smp", NULL, AD_NO_SIZE),
 			Common::DE_DEU,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -718,14 +771,14 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw2.scn", 0, "c6d15ce9720a9d8fef06e6582dcf3f34", 103593},
-				{"english1.smp", 0, NULL, -1},
+				{"english1.smp", 0, NULL, AD_NO_SIZE},
 				{"italian1.txt", 0, "d443249f8b55489b5888c227b9096f4e", 246495},
 				AD_LISTEND
 			},
 			Common::IT_ITA,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -738,14 +791,14 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw2.scn", 0, "c6d15ce9720a9d8fef06e6582dcf3f34", 103593},
-				{"english1.smp", 0, NULL, -1},
+				{"english1.smp", 0, NULL, AD_NO_SIZE},
 				{"spanish1.txt", 0, "bc6e147c5f542db228ac577357e4d897", 230323},
 				AD_LISTEND
 			},
 			Common::ES_ESP,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -759,14 +812,14 @@ static const TinselGameDescription gameDescriptions[] = {
 			"CD",
 			{
 				{"dw2.scn", 0, "c6d15ce9720a9d8fef06e6582dcf3f34", 103593},
-				{"english1.smp", 0, NULL, -1},
+				{"english1.smp", 0, NULL, AD_NO_SIZE},
 				{"english1.txt", 0, "b522e19d7b2cd7b85e50e36fe48e36a9", 274444},
 				AD_LISTEND
 			},
 			Common::RU_RUS,
 			Common::kPlatformDOS,
 			ADGF_NO_FLAGS,
-			GUIO1(GUIO_NOASPECT)
+			DISCWORLD2_GUIOPTIONS
 		},
 		GID_DW2,
 		0,
@@ -777,16 +830,17 @@ static const TinselGameDescription gameDescriptions[] = {
 	// ==== Discworld Noir entries ===============================================
 	// Note: All Discworld Noir versions are CD only, therefore we don't add the ADGF_CD flag
 
+#define NOIR_GUIOPTIONS GUIO1(GAMEOPTION_CROP_HEIGHT_480_TO_432)
 	{ // Discworld Noir, Windows 3CD version
 		{
 			"noir",
 			"CD",
-			AD_ENTRY2s("dw3.scn",		"16104acdc66cda903f860acac02a96bd", -1,
-					   "english.smp",	"94e510fd33c5c4a67b274bf5c068a87a", -1),
+			AD_ENTRY2s("dw3.scn",		"16104acdc66cda903f860acac02a96bd", AD_NO_SIZE,
+					   "english.smp",	"94e510fd33c5c4a67b274bf5c068a87a", AD_NO_SIZE),
 			Common::EN_ANY,
 			Common::kPlatformWindows,
 			ADGF_UNSTABLE,
-			GUIO0()
+			NOIR_GUIOPTIONS
 
 		},
 		GID_NOIR,
@@ -804,7 +858,7 @@ static const TinselGameDescription gameDescriptions[] = {
 			Common::EN_ANY,
 			Common::kPlatformWindows,
 			ADGF_UNSTABLE | ADGF_DEMO,
-			GUIO0()
+			NOIR_GUIOPTIONS
 		},
 		GID_NOIR,
 		0,
