@@ -4,19 +4,18 @@
  * are too numerous to list here. Please refer to the COPYRIGHT
  * file distributed with this source distribution.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
-
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -27,7 +26,7 @@
 #include "audio/decoders/raw.h"
 #include "audio/decoders/wave.h"
 
-#include "graphics/palette.h"
+#include "graphics/paletteman.h"
 #include "graphics/surface.h"
 
 #include "tucker/tucker.h"
@@ -40,7 +39,7 @@ void TuckerEngine::handleIntroSequence() {
 	_player = new AnimationSequencePlayer(_system, _mixer, _eventMan, &_compressedSound, firstSequence);
 	_player->mainLoop();
 	delete _player;
-	_player = 0;
+	_player = nullptr;
 }
 
 void TuckerEngine::handleCreditsSequence() {
@@ -50,12 +49,13 @@ void TuckerEngine::handleCreditsSequence() {
 	int counter3 = 0;
 	int num = 0;
 	int imgNum = 0;
-	int prevLocationNum = _locationNum;
+	Location prevLocation = _location;
 	int counter2 = 0;
 	int counter1 = 0;
 	loadCharset2();
+	showCursor(false);
 	stopSounds();
-	_locationNum = 74;
+	_location = kLocationCredits;
 	_flagsTable[236] = 74;
 	uint8 *imgBuf = (uint8 *)malloc(16 * 64000);
 	loadSprC02_01();
@@ -115,7 +115,8 @@ void TuckerEngine::handleCreditsSequence() {
 		if (counter4 == _creditsSequenceTimecounts[num]) {
 			_fadePaletteCounter = 0;
 			clearSprites();
-			++num;
+			if (num < 6)
+				++num;
 			Common::String filename;
 			if (num == 6) {
 				for (int i = 0; i < 16; ++i) {
@@ -123,6 +124,7 @@ void TuckerEngine::handleCreditsSequence() {
 					loadImage(filename.c_str(), imgBuf + i * 64000, 2);
 				}
 			} else {
+				filename = "";
 				switch (num) {
 				case 1:
 					filename = "loc75.pcx";
@@ -139,15 +141,18 @@ void TuckerEngine::handleCreditsSequence() {
 				case 5:
 					filename = "loc78.pcx";
 					break;
+				default:
+					break;
 				}
-				loadImage(filename.c_str(), _quadBackgroundGfxBuf, 2);
+				if (!filename.empty())
+					loadImage(filename.c_str(), _quadBackgroundGfxBuf, 2);
 			}
 			_spritesCount = _creditsSequenceSpriteCounts[num];
 			++_flagsTable[236];
 		}
 	} while (!_quitGame && isSpeechSoundPlaying());
 	free(imgBuf);
-	_locationNum = prevLocationNum;
+	_location = prevLocation;
 	do {
 		if (_fadePaletteCounter > 0) {
 			fadeInPalette();
@@ -156,12 +161,16 @@ void TuckerEngine::handleCreditsSequence() {
 		redrawScreen(0);
 		waitForTimer(2);
 	} while (_fadePaletteCounter > 0);
+	showCursor(true);
 }
 
 void TuckerEngine::handleCongratulationsSequence() {
+	// This method is only called right before the program terminates,
+	// so it doesn't bother restoring the palette
 	_timerCounter2 = 0;
 	_fadePaletteCounter = 0;
 	stopSounds();
+	showCursor(false);
 	loadImage("congrat.pcx", _loadTempBuf, 1);
 	Graphics::copyRect(_locationBackgroundGfxBuf, 640, _loadTempBuf, 320, 320, 200);
 	_fullRedraw = true;
@@ -173,11 +182,15 @@ void TuckerEngine::handleCongratulationsSequence() {
 		}
 		waitForTimer(3);
 	}
+	showCursor(true);
 }
 
 void TuckerEngine::handleNewPartSequence() {
-	char filename[40];
+	assert(_part != kPartInit);
 
+	Common::String filename;
+
+	showCursor(false);
 	stopSounds();
 	if (_flagsTable[219] == 1) {
 		_flagsTable[219] = 0;
@@ -187,43 +200,36 @@ void TuckerEngine::handleNewPartSequence() {
 		_inventoryObjectsOffset = 0;
 		_inventoryObjectsCount = 0;
 		addObjectToInventory(30);
-		if (_partNum == 1 || _partNum == 3) {
+		if (_part == kPartOne || _part == kPartThree) {
 			addObjectToInventory(1);
 			addObjectToInventory(0);
 		}
 		_redrawPanelItemsCounter = 0;
 	}
 	_scrollOffset = 0;
-	switch (_partNum) {
-	case 1:
-		strcpy(filename, "pt1bak.pcx");
-		break;
-	case 2:
-		strcpy(filename, "pt2bak.pcx");
-		break;
-	default:
-		strcpy(filename, "pt3bak.pcx");
-		break;
-	}
-	loadImage(filename, _quadBackgroundGfxBuf, 1);
+
+	filename = Common::String::format("pt%dbak.pcx", _part);
+	loadImage(filename.c_str(), _quadBackgroundGfxBuf, 1);
 	_spritesCount = 1;
 	clearSprites();
-	int currentLocation = _locationNum;
-	_locationNum = 98;
+	Location currentLocation = _location;
+	_location = kLocationNewPart;
 	unloadSprA02_01();
 	unloadSprC02_01();
-	switch (_partNum) {
-	case 1:
-		strcpy(filename, "sprites/partone.spr");
+	switch (_part) {
+	case kPartOne:
+		filename = "sprites/partone.spr";
 		break;
-	case 2:
-		strcpy(filename, "sprites/parttwo.spr");
+	case kPartTwo:
+		filename = "sprites/parttwo.spr";
+		break;
+	case kPartThree:
+		filename = "sprites/partthr.spr";
 		break;
 	default:
-		strcpy(filename, "sprites/partthr.spr");
 		break;
 	}
-	_sprC02Table[1] = loadFile(filename, 0);
+	_sprC02Table[1] = loadFile(filename.c_str(), nullptr);
 	startSpeechSound(9000, 60);
 	_fadePaletteCounter = 0;
 	do {
@@ -241,7 +247,7 @@ void TuckerEngine::handleNewPartSequence() {
 			_inputKeys[kInputKeyEscape] = false;
 			break;
 		}
-	} while (isSpeechSoundPlaying());
+	} while (isSpeechSoundPlaying() && !_quitGame);
 	stopSpeechSound();
 	do {
 		if (_fadePaletteCounter > 0) {
@@ -254,31 +260,25 @@ void TuckerEngine::handleNewPartSequence() {
 		drawSprite(0);
 		redrawScreen(0);
 		waitForTimer(3);
-	} while (_fadePaletteCounter > 0);
-	_locationNum = currentLocation;
+	} while (_fadePaletteCounter > 0 && !_quitGame);
+	_location = currentLocation;
+	showCursor(true);
 }
 
 void TuckerEngine::handleMeanwhileSequence() {
-	char filename[40];
+	assert(_part != kPartInit);
+
+	Common::String filename;
 	uint8 backupPalette[256 * 3];
 	memcpy(backupPalette, _currentPalette, 256 * 3);
-	switch (_partNum) {
-	case 1:
-		strcpy(filename, "meanw01.pcx");
-		break;
-	case 2:
-		strcpy(filename, "meanw02.pcx");
-		break;
-	default:
-		strcpy(filename, "meanw03.pcx");
-		break;
-	}
+	filename = Common::String::format("meanw%02d.pcx", _part);
 	if (_flagsTable[215] == 0 && _flagsTable[231] == 1) {
-		strcpy(filename, "loc80.pcx");
+		filename = "loc80.pcx";
 	}
-	loadImage(filename, _quadBackgroundGfxBuf + 89600, 1);
+	loadImage(filename.c_str(), _quadBackgroundGfxBuf + 89600, 1);
+	showCursor(false);
 	_fadePaletteCounter = 0;
-	for (int i = 0; i < 60; ++i) {
+	for (int i = 0; i < 60 && !_quitGame; ++i) {
 		if (_fadePaletteCounter < 16) {
 			fadeOutPalette();
 			++_fadePaletteCounter;
@@ -287,7 +287,10 @@ void TuckerEngine::handleMeanwhileSequence() {
 		_fullRedraw = true;
 		redrawScreen(0);
 		waitForTimer(3);
-		++i;
+		if (_inputKeys[kInputKeyEscape]) {
+			_inputKeys[kInputKeyEscape] = false;
+			break;
+		}
 	}
 	do {
 		if (_fadePaletteCounter > 0) {
@@ -298,15 +301,16 @@ void TuckerEngine::handleMeanwhileSequence() {
 		_fullRedraw = true;
 		redrawScreen(0);
 		waitForTimer(3);
-	} while (_fadePaletteCounter > 0);
+	} while (_fadePaletteCounter > 0 && !_quitGame);
 	memcpy(_currentPalette, backupPalette, 256 * 3);
 	_fullRedraw = true;
+	showCursor(true);
 }
 
 void TuckerEngine::handleMapSequence() {
 	loadImage("map2.pcx", _quadBackgroundGfxBuf + 89600, 0);
 	loadImage("map1.pcx", _loadTempBuf, 1);
-	_selectedObject.locationObject_locationNum = 0;
+	_selectedObject._locationObjectLocation = kLocationNone;
 	if (_flagsTable[7] > 0) {
 		copyMapRect(0, 0, 140, 86);
 	}
@@ -323,53 +327,54 @@ void TuckerEngine::handleMapSequence() {
 		copyMapRect(220, 0, 100, 180);
 	}
 	_fadePaletteCounter = 0;
-	int xPos = 0, yPos = 0, textNum = 0;
+	int xPos = 0, yPos = 0;
 	while (!_quitGame) {
+		int textNum = 0;
 		waitForTimer(2);
 		updateMouseState();
 		Graphics::copyRect(_locationBackgroundGfxBuf + _scrollOffset, 640, _quadBackgroundGfxBuf + 89600, 320, 320, 200);
 		_fullRedraw = true;
 		if (_flagsTable[7] > 0 && _mousePosX > 30 && _mousePosX < 86 && _mousePosY > 36 && _mousePosY < 86) {
 			textNum = 13;
-			_nextLocationNum = (_partNum == 1) ? 3 : 65;
+			_nextLocation = (_part == kPartOne) ? kLocationSeedyStreet : kLocationSeedyStreetPartThree;
 			xPos = 620;
 			yPos = 130;
 		} else if (_flagsTable[7] > 1 && _mousePosX > 60 && _mousePosX < 120 && _mousePosY > 120 && _mousePosY < 170) {
 			textNum = 14;
-			_nextLocationNum = (_partNum == 1) ? 9 : 66;
+			_nextLocation = (_part == kPartOne) ? kLocationMall : kLocationMallPartThree;
 			xPos = 344;
 			yPos = 120;
 		} else if (_flagsTable[7] > 2 && _mousePosX > 160 && _mousePosX < 210 && _mousePosY > 110 && _mousePosY < 160) {
 			textNum = 15;
-			_nextLocationNum = (_partNum == 1) ? 16 : 61;
+			_nextLocation = (_part == kPartOne) ? kLocationPark : kLocationParkPartThree;
 			xPos = 590;
 			yPos = 130;
 		} else if ((_flagsTable[7] == 4 || _flagsTable[7] == 6) && _mousePosX > 150 && _mousePosX < 200 && _mousePosY > 20 && _mousePosY < 70) {
 			textNum = 16;
-			_nextLocationNum = (_partNum == 1) ? 20 : 68;
+			_nextLocation = (_part == kPartOne) ? kLocationOutsideMuseum : kLocationOutsideMuseumPartThree;
 			xPos = 20;
 			yPos = 130;
 		} else if (_flagsTable[120] == 1 && _mousePosX > 240 && _mousePosX < 290 && _mousePosY > 35 && _mousePosY < 90) {
 			textNum = 17;
-			_nextLocationNum = (_partNum == 1) ? 19 : 62;
+			_nextLocation = (_part == kPartOne) ? kLocationDocks : kLocationDocksPartThree;
 			xPos = 20;
 			yPos = 124;
 		} else if (_mousePosX > 135 && _mousePosX < 185 && _mousePosY > 170 && _mousePosY < 200) {
 			textNum = 18;
-			_nextLocationNum = _locationNum;
+			_nextLocation = _location;
 			if (!_noPositionChangeAfterMap) {
 				xPos = _xPosCurrent;
 				yPos = _yPosCurrent;
-			} else if (_locationNum == 3 || _locationNum == 65) {
+			} else if (_location == kLocationSeedyStreet || _location == kLocationSeedyStreetPartThree) {
 				xPos = 620;
 				yPos = 130;
-			} else if (_locationNum == 9 || _locationNum == 66) {
+			} else if (_location == kLocationMall || _location == kLocationMallPartThree) {
 				xPos = 344;
 				yPos = 120;
-			} else if (_locationNum == 16 || _locationNum == 61) {
+			} else if (_location == kLocationPark || _location == kLocationParkPartThree) {
 				xPos = 590;
 				yPos = 130;
-			} else if (_locationNum == 20 || _locationNum == 68) {
+			} else if (_location == kLocationOutsideMuseum || _location == kLocationOutsideMuseumPartThree) {
 				xPos = 20;
 				yPos = 130;
 			} else {
@@ -395,14 +400,14 @@ void TuckerEngine::handleMapSequence() {
 		--_fadePaletteCounter;
 	}
 	_mouseClick = 1;
-	if (_nextLocationNum == 9 && _noPositionChangeAfterMap) {
+	if (_nextLocation == kLocationMall && _noPositionChangeAfterMap) {
 		_backgroundSpriteCurrentAnimation = 2;
 		_backgroundSpriteCurrentFrame = 0;
-		setCursorType(2);
-	} else if (_nextLocationNum == 66 && _noPositionChangeAfterMap) {
+		setCursorState(kCursorStateDisabledHidden);
+	} else if (_nextLocation == kLocationMallPartThree && _noPositionChangeAfterMap) {
 		_backgroundSpriteCurrentAnimation = 1;
 		_backgroundSpriteCurrentFrame = 0;
-		setCursorType(2);
+		setCursorState(kCursorStateDisabledHidden);
 	}
 	_noPositionChangeAfterMap = false;
 	_xPosCurrent = xPos;
@@ -419,24 +424,24 @@ void TuckerEngine::copyMapRect(int x, int y, int w, int h) {
 	}
 }
 
-int TuckerEngine::handleSpecialObjectSelectionSequence() {
-	char filename[40];
-	if (_partNum == 1 && _selectedObjectNum == 6) {
-		strcpy(filename, "news1.pcx");
+bool TuckerEngine::handleSpecialObjectSelectionSequence() {
+	Common::String filename;
+	if (_part == kPartOne && _selectedObjectNum == 6) {
+		filename = "news1.pcx";
 		_flagsTable[7] = 4;
-	} else if (_partNum == 3 && _selectedObjectNum == 45) {
-		strcpy(filename, "profnote.pcx");
-	} else if (_partNum == 1 && _selectedObjectNum == 26) {
-		strcpy(filename, "photo.pcx");
-	} else if (_partNum == 3 && _selectedObjectNum == 39) {
-		strcpy(filename, "news2.pcx");
+	} else if (_part == kPartThree && _selectedObjectNum == 45) {
+		filename = "profnote.pcx";
+	} else if (_part == kPartOne && _selectedObjectNum == 26) {
+		filename = "photo.pcx";
+	} else if (_part == kPartThree && _selectedObjectNum == 39) {
+		filename = "news2.pcx";
 		_flagsTable[135] = 1;
 	} else if (_currentInfoString1SourceType == 0 && _currentActionObj1Num == 259) {
-		strcpy(filename, "postit.pcx");
+		filename = "postit.pcx";
 	} else if (_currentInfoString1SourceType == 1 && _currentActionObj1Num == 91) {
-		strcpy(filename, "memo.pcx");
+		filename = "memo.pcx";
 	} else {
-		return 0;
+		return false;
 	}
 	while (_fadePaletteCounter > 0) {
 		fadeInPalette();
@@ -444,7 +449,7 @@ int TuckerEngine::handleSpecialObjectSelectionSequence() {
 		--_fadePaletteCounter;
 	}
 	_mouseClick = 1;
-	loadImage(filename, _quadBackgroundGfxBuf, 1);
+	loadImage(filename.c_str(), _quadBackgroundGfxBuf, 1);
 	_fadePaletteCounter = 0;
 	while (!_quitGame) {
 		waitForTimer(2);
@@ -458,12 +463,12 @@ int TuckerEngine::handleSpecialObjectSelectionSequence() {
 		if (!_leftMouseButtonPressed && _mouseClick == 1) {
 			_mouseClick = 0;
 		}
-		if (_partNum == 3 && _selectedObjectNum == 45) {
+		if (_part == kPartThree && _selectedObjectNum == 45) {
 			for (int i = 0; i < 13; ++i) {
-				const int offset = _dataTable[204 + i].yDest * 640 + _dataTable[204 + i].xDest;
+				const int offset = _dataTable[204 + i]._yDest * 640 + _dataTable[204 + i]._xDest;
 				static const int itemsTable[] = { 15, 44, 25, 19, 21, 24, 12, 27, 20, 29, 35, 23, 3 };
 				if (_inventoryItemsState[itemsTable[i]] > 1) {
-					Graphics::decodeRLE(_locationBackgroundGfxBuf + _scrollOffset + offset, _data3GfxBuf + _dataTable[204 + i].sourceOffset, _dataTable[204 + i].xSize, _dataTable[204 + i].ySize);
+					Graphics::decodeRLE(_locationBackgroundGfxBuf + _scrollOffset + offset, _data3GfxBuf + _dataTable[204 + i]._sourceOffset, _dataTable[204 + i]._xSize, _dataTable[204 + i]._ySize);
 				}
 			}
 		}
@@ -479,7 +484,7 @@ int TuckerEngine::handleSpecialObjectSelectionSequence() {
 		}
 	}
 	loadLoc();
-	return 1;
+	return true;
 }
 
 AnimationSequencePlayer::AnimationSequencePlayer(OSystem *system, Audio::Mixer *mixer, Common::EventManager *event, CompressedSound *sound, int num)
@@ -487,11 +492,20 @@ AnimationSequencePlayer::AnimationSequencePlayer(OSystem *system, Audio::Mixer *
 	memset(_animationPalette, 0, sizeof(_animationPalette));
 	_soundSeqDataCount = 0;
 	_soundSeqDataIndex = 0;
-	_soundSeqData = 0;
+	_soundSeqData = nullptr;
 	_offscreenBuffer = (uint8 *)malloc(kScreenWidth * kScreenHeight);
 	_updateScreenWidth = 0;
 	_updateScreenPicture = false;
-	_picBufPtr = _pic2BufPtr = 0;
+	_picBufPtr = _pic2BufPtr = nullptr;
+
+	_changeToNextSequence = false;
+	_updateFunc = nullptr;
+	_updateFuncIndex = 0;
+	_updateScreenCounter = 0;
+	_updateScreenIndex = -1;
+	_frameCounter = 0;
+	_frameTime = 0;
+	_lastFrameTime = 1;
 }
 
 AnimationSequencePlayer::~AnimationSequencePlayer() {
@@ -504,7 +518,7 @@ void AnimationSequencePlayer::mainLoop() {
 		{ 13, 2, &AnimationSequencePlayer::loadIntroSeq13_14, &AnimationSequencePlayer::playIntroSeq13_14 },
 		{ 15, 2, &AnimationSequencePlayer::loadIntroSeq15_16, &AnimationSequencePlayer::playIntroSeq15_16 },
 		{ 27, 2, &AnimationSequencePlayer::loadIntroSeq27_28, &AnimationSequencePlayer::playIntroSeq27_28 },
-		{  1, 0, 0, 0 }
+		{  1, 0, nullptr,                                     nullptr                                     }
 	};
 	static const SequenceUpdateFunc _gameSeqUpdateFuncs[] = {
 		{ 17, 1, &AnimationSequencePlayer::loadIntroSeq17_18, &AnimationSequencePlayer::playIntroSeq17_18 },
@@ -512,7 +526,7 @@ void AnimationSequencePlayer::mainLoop() {
 		{  3, 2, &AnimationSequencePlayer::loadIntroSeq3_4,   &AnimationSequencePlayer::playIntroSeq3_4   },
 		{  9, 2, &AnimationSequencePlayer::loadIntroSeq9_10,  &AnimationSequencePlayer::playIntroSeq9_10  },
 		{ 21, 2, &AnimationSequencePlayer::loadIntroSeq21_22, &AnimationSequencePlayer::playIntroSeq21_22 },
-		{  1, 0, 0, 0 }
+		{  1, 0, nullptr,                                     nullptr                                     }
 	};
 	switch (_seqNum) {
 	case kFirstAnimationSequenceDemo:
@@ -521,6 +535,8 @@ void AnimationSequencePlayer::mainLoop() {
 	case kFirstAnimationSequenceGame:
 		_updateFunc = _gameSeqUpdateFuncs;
 		break;
+	default:
+		break;
 	}
 	_updateFuncIndex = 0;
 	_changeToNextSequence = true;
@@ -562,13 +578,13 @@ void AnimationSequencePlayer::syncTime() {
 		Common::Event ev;
 		while (_event->pollEvent(ev)) {
 			switch (ev.type) {
-			case Common::EVENT_KEYDOWN:
-				if (ev.kbd.keycode == Common::KEYCODE_ESCAPE) {
+			case Common::EVENT_CUSTOM_ENGINE_ACTION_START:
+				if (ev.customType == kActionEscape) {
 					_seqNum = 1;
 				}
 				break;
 			case Common::EVENT_QUIT:
-			case Common::EVENT_RTL:
+			case Common::EVENT_RETURN_TO_LAUNCHER:
 				_seqNum = 1;
 				break;
 			default:
@@ -585,15 +601,15 @@ Audio::RewindableAudioStream *AnimationSequencePlayer::loadSound(int index, Anim
 	if (stream)
 		return stream;
 
-	Common::String fileName = Common::String::format("audio/%s", _audioFileNamesTable[index]);
-	Common::File f;
-	if (f.open(fileName)) {
+	Common::Path fileName(Common::String::format("audio/%s", _audioFileNamesTable[index]));
+	Common::File *f = new Common::File();
+	if (f->open(fileName)) {
 		int size = 0, rate = 0;
 		uint8 flags = 0;
 		switch (type) {
 		case kAnimationSoundType8BitsRAW:
 		case kAnimationSoundType16BitsRAW:
-			size = f.size();
+			size = f->size();
 			rate = 22050;
 			flags = Audio::FLAG_UNSIGNED;
 			if (type == kAnimationSoundType16BitsRAW)
@@ -602,16 +618,21 @@ Audio::RewindableAudioStream *AnimationSequencePlayer::loadSound(int index, Anim
 			if (size != 0) {
 				uint8 *sampleData = (uint8 *)malloc(size);
 				if (sampleData) {
-					f.read(sampleData, size);
+					f->read(sampleData, size);
 					stream = Audio::makeRawStream(sampleData, size, rate, flags);
 				}
 			}
+			delete f;
 			break;
 		case kAnimationSoundTypeWAV:
-			stream = Audio::makeWAVStream(&f, DisposeAfterUse::NO);
+			stream = Audio::makeWAVStream(f, DisposeAfterUse::YES);
+			break;
+		default:
+			delete f;
 			break;
 		}
-
+	} else {
+		delete f;
 	}
 	return stream;
 }
@@ -619,7 +640,7 @@ Audio::RewindableAudioStream *AnimationSequencePlayer::loadSound(int index, Anim
 void AnimationSequencePlayer::loadSounds(int num) {
 	if (_soundSeqDataList[num].musicVolume != 0) {
 		Audio::AudioStream *s;
-		if ((s = loadSound(_soundSeqDataList[num].musicIndex, kAnimationSoundType8BitsRAW)) != 0) {
+		if ((s = loadSound(_soundSeqDataList[num].musicIndex, kAnimationSoundType8BitsRAW)) != nullptr) {
 			_mixer->playStream(Audio::Mixer::kMusicSoundType, &_musicHandle, s, -1, scaleMixerVolume(_soundSeqDataList[num].musicVolume));
 		}
 	}
@@ -629,17 +650,17 @@ void AnimationSequencePlayer::loadSounds(int num) {
 }
 
 void AnimationSequencePlayer::updateSounds() {
-	Audio::RewindableAudioStream *s = 0;
+	Audio::RewindableAudioStream *s = nullptr;
 	const SoundSequenceData *p = &_soundSeqData[_soundSeqDataIndex];
 	while (_soundSeqDataIndex < _soundSeqDataCount && p->timestamp <= _frameCounter) {
 		switch (p->opcode) {
 		case 0:
-			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != 0) {
+			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != nullptr) {
 				_mixer->playStream(Audio::Mixer::kSFXSoundType, &_soundsHandle[p->index], s, -1, scaleMixerVolume(p->volume));
 			}
 			break;
 		case 1:
-			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != 0) {
+			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != nullptr) {
 				_mixer->playStream(Audio::Mixer::kSFXSoundType, &_soundsHandle[p->index], Audio::makeLoopingAudioStream(s, 0),
 				                        -1, scaleMixerVolume(p->volume));
 			}
@@ -652,18 +673,18 @@ void AnimationSequencePlayer::updateSounds() {
 			break;
 		case 4:
 			_mixer->stopHandle(_musicHandle);
-			if ((s = loadSound(p->num, kAnimationSoundType8BitsRAW)) != 0) {
+			if ((s = loadSound(p->num, kAnimationSoundType8BitsRAW)) != nullptr) {
 				_mixer->playStream(Audio::Mixer::kMusicSoundType, &_musicHandle, s, -1, scaleMixerVolume(p->volume));
 			}
 			break;
 		case 5:
-			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != 0) {
+			if ((s = loadSound(p->num, kAnimationSoundTypeWAV)) != nullptr) {
 				_mixer->playStream(Audio::Mixer::kSFXSoundType, &_sfxHandle, s, -1, scaleMixerVolume(p->volume));
 			}
 			break;
 		case 6:
 			_mixer->stopHandle(_musicHandle);
-			if ((s = loadSound(p->num, kAnimationSoundType16BitsRAW)) != 0) {
+			if ((s = loadSound(p->num, kAnimationSoundType16BitsRAW)) != nullptr) {
 				_mixer->playStream(Audio::Mixer::kMusicSoundType, &_musicHandle, s, -1, scaleMixerVolume(p->volume));
 			}
 			break;
@@ -722,13 +743,12 @@ void AnimationSequencePlayer::fadeOutPalette() {
 void AnimationSequencePlayer::unloadAnimation() {
 	_mixer->stopAll();
 	free(_picBufPtr);
-	_picBufPtr = 0;
 	free(_pic2BufPtr);
-	_pic2BufPtr = 0;
+	_picBufPtr = _pic2BufPtr = nullptr;
 }
 
 uint8 *AnimationSequencePlayer::loadPicture(const char *fileName) {
-	uint8 *p = 0;
+	uint8 *p = nullptr;
 	Common::File f;
 	if (f.open(fileName)) {
 		const int sz = f.size();
@@ -763,7 +783,7 @@ bool AnimationSequencePlayer::decodeNextAnimationFrame(int index, bool copyDirty
 
 	if (!copyDirtyRects) {
 		for (uint16 y = 0; (y < surface->h) && (y < kScreenHeight); y++)
-			memcpy(_offscreenBuffer + y * kScreenWidth, (byte *)surface->pixels + y * surface->pitch, surface->w);
+			memcpy(_offscreenBuffer + y * kScreenWidth, (const byte *)surface->getBasePtr(0, y), surface->w);
 	} else {
 		_flicPlayer[index].copyDirtyRectsToBuffer(_offscreenBuffer, kScreenWidth);
 	}
@@ -798,7 +818,7 @@ void AnimationSequencePlayer::playIntroSeq19_20() {
 	// The intro credits animation. This uses 2 animations: the foreground one, which
 	// is the actual intro credits, and the background one, which is an animation of
 	// cogs, and is being replayed when an intro credit appears
-	const ::Graphics::Surface *surface = 0;
+	const ::Graphics::Surface *surface = nullptr;
 
 	if (_flicPlayer[0].getCurFrame() >= 115) {
 		surface = _flicPlayer[1].decodeNextFrame();
@@ -811,7 +831,7 @@ void AnimationSequencePlayer::playIntroSeq19_20() {
 	if (surface)
 		for (int i = 0; i < kScreenWidth * kScreenHeight; ++i)
 			if (_offscreenBuffer[i] == 0)
-				_offscreenBuffer[i] = *((byte *)surface->pixels + i);
+				_offscreenBuffer[i] = *((const byte *)surface->getPixels() + i);
 
 	if (!framesLeft)
 		_changeToNextSequence = true;
